@@ -55,6 +55,19 @@ static struct json_value *json_create_value_int(long long number)
 	return value;
 }
 
+static struct json_value *json_create_value_uint(unsigned long long number)
+{
+	struct json_value *value = malloc(sizeof(struct json_value));
+
+	if (value) {
+		value->type = JSON_TYPE_UINT;
+		value->uint_number = number;
+	} else
+		fail_and_notify();
+
+	return value;
+}
+
 static struct json_value *json_create_value_float(long double number)
 {
 	struct json_value *value = malloc(sizeof(struct json_value));
@@ -110,6 +123,7 @@ static struct json_value *json_create_value_string(const char *str)
 		if (!value->string) {
 			free(value);
 			value = NULL;
+			return value;
 		}
 	}
 	if (!value)
@@ -234,6 +248,8 @@ int json_object_add_value_type(struct json_object *obj, const char *name, int ty
 		value = json_create_value_string(va_arg(args, char *));
 	else if (type == JSON_TYPE_INTEGER)
 		value = json_create_value_int(va_arg(args, long long));
+	else if (type == JSON_TYPE_UINT)
+		value = json_create_value_uint(va_arg(args, unsigned long long));
 	else if (type == JSON_TYPE_FLOAT)
 		value = json_create_value_float(va_arg(args, long double));
 	else if (type == JSON_TYPE_OBJECT)
@@ -270,6 +286,8 @@ int json_array_add_value_type(struct json_array *array, int type, ...)
 		value = json_create_value_string(va_arg(args, char *));
 	else if (type == JSON_TYPE_INTEGER)
 		value = json_create_value_int(va_arg(args, long long));
+	else if (type == JSON_TYPE_UINT)
+		value = json_create_value_uint(va_arg(args, unsigned long long));
 	else if (type == JSON_TYPE_FLOAT)
 		value = json_create_value_float(va_arg(args, double));
 	else if (type == JSON_TYPE_OBJECT)
@@ -373,6 +391,9 @@ static void json_print_value(struct json_value *value, void *out)
 	case JSON_TYPE_INTEGER:
 		printf( "%lld", value->integer_number);
 		break;
+	case JSON_TYPE_UINT:
+		printf( "%llu", value->uint_number);
+		break;
 	case JSON_TYPE_FLOAT:
 		printf( "%.0Lf", value->float_number);
 		break;
